<#
.SYNOPSIS
    Creates a package containing Python for Dataiku Science Studio.
.DESCRIPTION
    Builds a package containing Python for Dataiku Science Studio by building python from the sources.
.PARAMETER Version
    (Optional) The python version to build.
.PARAMETER OutputDir
    (Optional) The output directory, by default './python-win'.
.EXAMPLE
    C:\PS> ./build-embbeded-python.ps1 -Version 3.10.16
    C:\PS> ./build-embbeded-python.ps1 -Version 3.10.16 -OutputDir "C:/dev"
#>
param(
    [Parameter(Mandatory=$false)] [string] $Version = "3.11.11",
    [Parameter(Mandatory=$false)] [string] $OutputDir = "./python-win"
)

$tagsResponse = Invoke-RestMethod -Uri "https://api.github.com/repos/python/cpython/git/refs/tags"
$tags = [System.Linq.Enumerable]::Select($tagsResponse, [Func[object, string]] { param($tag) $tag.ref.TrimStart("refs/tags/v") } )
$tags = [System.Linq.Enumerable]::ToList($tags)

if (!$tags.Contains("$Version"))
{
    Write-Error "The given version '$Version' does not exist. Specify one of the following versions (without the v): $tags"
    exit 1
}

$TmpDir = New-Item -ItemType Directory -Path (Join-Path ("./") ([System.Guid]::NewGuid()))
Write-Host "Created temporary directory: '$TmpDir'"

# Install build tools

$VSBuildToolsUrl = "https://aka.ms/vs/17/release/vs_buildtools.exe"
$VSBuildToolsFile = "$TmpDir/vs_BuildTools.exe"

Invoke-WebRequest $VSBuildToolsUrl -OutFile $VSBuildToolsFile

Write-Host "Downloaded $VSBuildToolsFile"

# We have to use the Start-Process command to wait for the installer process to finish
Start-Process -FilePath $VSBuildToolsFile -ArgumentList "--includeRecommended", "--add", "Microsoft.VisualStudio.Workload.VCTools", "Microsoft.VisualStudio.Component.Windows10SDK", "Microsoft.VisualStudio.Component.Windows10SDK.19041", "Microsoft.VisualStudio.Component.VC.140", "Microsoft.VisualStudio.Workload.MSBuildTools", "Microsoft.VisualStudio.Component.VC.CLI.Support", "--passive", "--wait" -Wait

Write-Host "Installed required VS Build tools."

# Refresh the Path to include the newly installed MSBuild
$env:Path += ";C:\Program Files (x86)\Microsoft Visual Studio\2022\BuildTools\MSBuild\Current\Bin"

$repoArchiveUrl = "http://github.com/python/cpython/archive/v$Version.zip"
$repoArchivePath = "$TmpDir/cpython-$Version.zip"

Invoke-WebRequest $repoArchiveUrl -OutFile $repoArchivePath

Write-Host "Downloaded $repoArchivePath"

Expand-Archive $repoArchivePath -DestinationPath $TmpDir

$repoPath = "$TmpDir/cpython-$Version"

Write-Host "Extracted repo to $repoPath"

& "$repoPath/Tools/nuget/build.bat" "-x64"

$NugetFile = "$repoPath/PCBuild/amd64/python.$Version.nupkg"
$NugetZip = "$repoPath/PCBuild/amd64/python.$Version.zip"

Move-Item -Path $NugetFile -Destination $NugetZip

Expand-Archive -Path $NugetZip -DestinationPath $TmpDir

# As stated in the following doc we can redistribute these files: https://learn.microsoft.com/en-us/visualstudio/releases/2022/redistribution#visual-c-runtime-files
$redistDlls = (Get-ChildItem -Path "C:\Program Files (x86)\Microsoft Visual Studio\2022\BuildTools\VC\Redist\MSVC\*\x64\*\*.dll" -File).FullName
Copy-Item $redistDlls -Destination "$TmpDir/tools/"

mkdir -Force $OutputDir | Out-Null

# We have to include this script in the archive we ship
Copy-Item "$PSCommandPath" -Destination "$TmpDir/tools/"

$PythonDir = "$TmpDir/python-$Version"

Move-Item -Path "$TmpDir/tools" -Destination $PythonDir

Compress-Archive -Force -Path $PythonDir -DestinationPath "$OutputDir/python-$Version-win.zip"

Write-Host "$OutputDir/python-$Version-win.zip"

Remove-Item -LiteralPath $TmpDir -Force -Recurse

Write-Host "Deleted '$TmpDir'"
